@@ -145,6 +145,8 @@ def drop_all_ip_traffic_for_port(port):
                "priority": ovn_const.ACL_PRIORITY_DROP,
                "action": ovn_const.ACL_ACTION_DROP,
                "log": False,
+               "name": [],
+               "severity": [],
                "direction": direction,
                "match": '%s == "%s" && ip' % (p, port['id']),
                "external_ids": {'neutron:lport': port['id']}}
@@ -162,9 +164,12 @@ def add_sg_rule_acl_for_port(port, r, match):
            "priority": ovn_const.ACL_PRIORITY_ALLOW,
            "action": ovn_const.ACL_ACTION_ALLOW_RELATED,
            "log": False,
+           "name": [],
+           "severity": [],
            "direction": dir_map[r['direction']],
            "match": match,
-           "external_ids": {'neutron:lport': port['id']}}
+           "external_ids": {'neutron:lport': port['id'],
+                            ovn_const.OVN_SG_RULE_EXT_ID_KEY: r['id']}}
     return acl
 
 
@@ -180,6 +185,8 @@ def add_acl_dhcp(port, subnet, ovn_dhcp=True):
                "priority": ovn_const.ACL_PRIORITY_ALLOW,
                "action": ovn_const.ACL_ACTION_ALLOW,
                "log": False,
+               "name": [],
+               "severity": [],
                "direction": 'to-lport',
                "match": ('outport == "%s" && ip4 && ip4.src == %s && '
                          'udp && udp.src == 67 && udp.dst == 68'
@@ -191,6 +198,8 @@ def add_acl_dhcp(port, subnet, ovn_dhcp=True):
            "priority": ovn_const.ACL_PRIORITY_ALLOW,
            "action": ovn_const.ACL_ACTION_ALLOW,
            "log": False,
+           "name": [],
+           "severity": [],
            "direction": 'from-lport',
            "match": ('inport == "%s" && ip4 && '
                      'ip4.dst == {255.255.255.255, %s} && '
@@ -266,6 +275,11 @@ def _add_sg_rule_acl_for_port(port, r):
     return add_sg_rule_acl_for_port(port, r, match)
 
 
+def _acl_columns_name_severity_supported(nb_idl):
+    columns = list(nb_idl._tables['ACL'].columns)
+    return ('name' in columns) and ('severity' in columns)
+
+
 def update_acls_for_security_group(plugin,
                                    admin_context,
                                    ovn,
@@ -289,20 +303,30 @@ def update_acls_for_security_group(plugin,
     sg_port_ids = list(set(sg_port_ids))
     port_list = plugin.get_ports(admin_context,
                                  filters={'id': sg_port_ids})
+    if not port_list:
+        return
+
     acl_new_values_dict = {}
     update_port_list = []
 
+    # Check if ACL log name and severity supported or not
+    keep_name_severity = _acl_columns_name_severity_supported(ovn)
     # NOTE(lizk): We can directly locate the affected acl records,
     # so no need to compare new acl values with existing acl objects.
     for port in port_list:
         # Skip trusted port
         if utils.is_lsp_trusted(port):
             continue
+
         update_port_list.append(port)
         acl = _add_sg_rule_acl_for_port(port, security_group_rule)
         # Remove lport and lswitch since we don't need them
         acl.pop('lport')
         acl.pop('lswitch')
+        # Remove ACL log name and severity if not supported,
+        if not keep_name_severity:
+            acl.pop('name')
+            acl.pop('severity')
         acl_new_values_dict[port['id']] = acl
 
     if not update_port_list:
@@ -316,7 +340,7 @@ def update_acls_for_security_group(plugin,
                     is_add_acl=is_add_acl).execute(check_error=True)
 
 
-def add_acls(plugin, admin_context, port, sg_cache, subnet_cache):
+def add_acls(plugin, admin_context, port, sg_cache, subnet_cache, ovn):
     acl_list = []
 
     # Skip ACLs if security groups aren't enabled
@@ -353,8 +377,13 @@ def add_acls(plugin, admin_context, port, sg_cache, subnet_cache):
                                 sg_id)
         for r in sg['security_group_rules']:
             acl = _add_sg_rule_acl_for_port(port, r)
-            if acl not in acl_list:
-                acl_list.append(acl)
+            acl_list.append(acl)
+
+    # Remove ACL log name and severity if not supported,
+    if not _acl_columns_name_severity_supported(ovn):
+        for acl in acl_list:
+            acl.pop('name')
+            acl.pop('severity')
 
     return acl_list
 
@@ -364,10 +393,6 @@ def acl_port_ips(port):
     if not is_sg_enabled():
         return {'ip4': [], 'ip6': []}
 
-    ip_addresses = {4: [], 6: []}
-    for fixed_ip in port['fixed_ips']:
-        ip_version = netaddr.IPNetwork(fixed_ip['ip_address']).version
-        ip_addresses[ip_version].append(fixed_ip['ip_address'])
-
-    return {'ip4': ip_addresses[4],
-            'ip6': ip_addresses[6]}
+    ip_list = [x['ip_address'] for x in port.get('fixed_ips', [])]
+    ip_list.extend(utils.get_allowed_address_pairs_ip_addresses(port))
+    return utils.sort_ips_by_version(ip_list)
