@@ -15,11 +15,11 @@
 import itertools
 import shlex
 
-from neutron._i18n import _
 from neutron_lib.utils import host
 from oslo_config import cfg
 from oslo_privsep import priv_context
 
+from networking_ovn._i18n import _
 
 DEDUCE_MODE = 'deduce'
 USER_MODE = 'user'
