@@ -15,7 +15,6 @@
 import threading
 
 from neutron_lib.api.definitions import portbindings
-from neutron_lib.api.definitions import provider_net as pnet
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
@@ -37,8 +36,10 @@ from networking_ovn._i18n import _
 from networking_ovn.common import acl as ovn_acl
 from networking_ovn.common import config
 from networking_ovn.common import constants as ovn_const
+from networking_ovn.common import maintenance
 from networking_ovn.common import ovn_client
 from networking_ovn.common import utils
+from networking_ovn.db import revision as db_rev
 from networking_ovn.ml2 import qos_driver
 from networking_ovn.ml2 import trunk_driver
 from networking_ovn import ovn_db_sync
@@ -93,14 +94,14 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._sb_ovn = None
         self._plugin_property = None
         self._ovn_client_inst = None
+        self._maintenance_thread = None
         self.sg_enabled = ovn_acl.is_sg_enabled()
         self._post_fork_event = threading.Event()
         if cfg.CONF.SECURITYGROUP.firewall_driver:
             LOG.warning('Firewall driver configuration is ignored')
         self._setup_vif_port_bindings()
         self.subscribe()
-        qos_driver.OVNQosNotificationDriver.create()
-        self.qos_driver = qos_driver.OVNQosDriver(self)
+        self.qos_driver = qos_driver.OVNQosNotificationDriver.create(self)
         self.trunk_driver = trunk_driver.OVNTrunkDriver.create(self)
 
     @property
@@ -120,14 +121,9 @@ class OVNMechanismDriver(api.MechanismDriver):
                                                          self._sb_ovn)
         return self._ovn_client_inst
 
-    def _get_attribute(self, obj, attribute):
-        res = obj.get(attribute)
-        if res is const.ATTR_NOT_SPECIFIED:
-            res = None
-        return res
-
     def _setup_vif_port_bindings(self):
-        self.supported_vnic_types = [portbindings.VNIC_NORMAL]
+        self.supported_vnic_types = [portbindings.VNIC_NORMAL,
+                                     portbindings.VNIC_DIRECT]
         self.vif_details = {
             portbindings.VIF_TYPE_OVS: {
                 portbindings.CAP_PORT_FILTER: self.sg_enabled
@@ -151,15 +147,21 @@ class OVNMechanismDriver(api.MechanismDriver):
 
         # Handle security group/rule notifications
         if self.sg_enabled:
-            registry.subscribe(self._process_sg_notification,
+            registry.subscribe(self._create_security_group_precommit,
                                resources.SECURITY_GROUP,
-                               events.AFTER_CREATE)
-            registry.subscribe(self._process_sg_notification,
+                               events.PRECOMMIT_CREATE)
+            registry.subscribe(self._update_security_group,
                                resources.SECURITY_GROUP,
                                events.AFTER_UPDATE)
-            registry.subscribe(self._process_sg_notification,
+            registry.subscribe(self._create_security_group,
                                resources.SECURITY_GROUP,
-                               events.BEFORE_DELETE)
+                               events.AFTER_CREATE)
+            registry.subscribe(self._delete_security_group,
+                               resources.SECURITY_GROUP,
+                               events.AFTER_DELETE)
+            registry.subscribe(self._create_sg_rule_precommit,
+                               resources.SECURITY_GROUP_RULE,
+                               events.PRECOMMIT_CREATE)
             registry.subscribe(self._process_sg_rule_notification,
                                resources.SECURITY_GROUP_RULE,
                                events.AFTER_CREATE)
@@ -167,12 +169,15 @@ class OVNMechanismDriver(api.MechanismDriver):
                                resources.SECURITY_GROUP_RULE,
                                events.BEFORE_DELETE)
 
-    def post_fork_initialize(self, resource, event, trigger, **kwargs):
+    def post_fork_initialize(self, resource, event, trigger, payload=None):
         # NOTE(rtheis): This will initialize all workers (API, RPC,
         # plugin service and OVN) with OVN IDL connections.
         self._post_fork_event.clear()
+        self._ovn_client_inst = None
         self._nb_ovn, self._sb_ovn = impl_idl_ovn.get_ovn_idls(self,
                                                                trigger)
+        # Now IDL connections can be safely used.
+        self._post_fork_event.set()
 
         if trigger.im_class == ovsdb_monitor.OvnWorker:
             # Call the synchronization task if its ovn worker
@@ -180,6 +185,7 @@ class OVNMechanismDriver(api.MechanismDriver):
             self.nb_synchronizer = ovn_db_sync.OvnNbSynchronizer(
                 self._plugin,
                 self._nb_ovn,
+                self._sb_ovn,
                 config.get_ovn_neutron_sync_mode(),
                 self
             )
@@ -193,17 +199,39 @@ class OVNMechanismDriver(api.MechanismDriver):
             )
             self.sb_synchronizer.sync()
 
-        self._post_fork_event.set()
+        if trigger.im_class == maintenance.MaintenanceWorker:
+            self._maintenance_thread = maintenance.MaintenanceThread()
+            self._maintenance_thread.add_periodics(
+                maintenance.DBInconsistenciesPeriodics(self._ovn_client))
+            self._maintenance_thread.start()
 
-    def _process_sg_notification(self, resource, event, trigger, **kwargs):
-        sg = {'name': kwargs['security_group']['name'],
-              'id': kwargs['security_group']['id']}
-        if event == events.AFTER_CREATE:
-            self._ovn_client.create_security_group(sg)
-        elif event == events.AFTER_UPDATE:
-            self._ovn_client.update_security_group(sg)
-        elif event == events.BEFORE_DELETE:
-            self._ovn_client.delete_security_group(sg)
+    def _create_security_group_precommit(self, resource, event, trigger,
+                                         security_group, context, **kwargs):
+        db_rev.create_initial_revision(
+            security_group['id'], ovn_const.TYPE_SECURITY_GROUPS,
+            context.session)
+
+    def _create_security_group(self, resource, event, trigger,
+                               security_group, **kwargs):
+        self._ovn_client.create_security_group(security_group)
+
+    def _delete_security_group(self, resource, event, trigger,
+                               security_group_id, **kwargs):
+        self._ovn_client.delete_security_group(security_group_id)
+
+    def _update_security_group(self, resource, event, trigger,
+                               security_group, **kwargs):
+        # OVN doesn't care about updates to security groups, only if they
+        # exist or not. We are bumping the revision number here so it
+        # doesn't show as inconsistent to the maintenance periodic task
+        db_rev.bump_revision(security_group, ovn_const.TYPE_SECURITY_GROUPS)
+
+    def _create_sg_rule_precommit(self, resource, event, trigger, **kwargs):
+        sg_rule = kwargs.get('security_group_rule')
+        context = kwargs.get('context')
+        db_rev.create_initial_revision(sg_rule['id'],
+                                       ovn_const.TYPE_SECURITY_GROUP_RULES,
+                                       context.session)
 
     def _process_sg_rule_notification(
             self, resource, event, trigger, **kwargs):
@@ -250,6 +278,9 @@ class OVNMechanismDriver(api.MechanismDriver):
         of the current transaction.
         """
         self._validate_network_segments(context.network_segments)
+        db_rev.create_initial_revision(
+            context.current['id'], ovn_const.TYPE_NETWORKS,
+            context._plugin_context.session)
 
     def create_network_postcommit(self, context):
         """Create a network.
@@ -263,9 +294,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         cause the deletion of the resource.
         """
         network = context.current
-        physnet = self._get_attribute(network, pnet.PHYSICAL_NETWORK)
-        segid = self._get_attribute(network, pnet.SEGMENTATION_ID)
-        self._ovn_client.create_network(network, physnet, segid)
+        self._ovn_client.create_network(network)
 
     def update_network_precommit(self, context):
         """Update resources of a network.
@@ -301,10 +330,11 @@ class OVNMechanismDriver(api.MechanismDriver):
         network state.  It is up to the mechanism driver to ignore
         state or state changes that it does not know or care about.
         """
-        network = context.current
-        original_network = context.original
-        if network['name'] != original_network['name']:
-            self._ovn_client.update_network(network, original_network)
+        # FIXME(lucasagomes): We can delete this conditional after
+        # https://bugs.launchpad.net/neutron/+bug/1739798 is fixed.
+        if context._plugin_context.session.is_active:
+            return
+        self._ovn_client.update_network(context.current)
 
     def delete_network_postcommit(self, context):
         """Delete a network.
@@ -320,13 +350,18 @@ class OVNMechanismDriver(api.MechanismDriver):
         """
         self._ovn_client.delete_network(context.current['id'])
 
+    def create_subnet_precommit(self, context):
+        db_rev.create_initial_revision(
+            context.current['id'], ovn_const.TYPE_SUBNETS,
+            context._plugin_context.session)
+
     def create_subnet_postcommit(self, context):
         self._ovn_client.create_subnet(context.current,
                                        context.network.current)
 
     def update_subnet_postcommit(self, context):
-        self._ovn_client.update_subnet(context.current, context.original,
-                                       context.network.current)
+        self._ovn_client.update_subnet(
+            context.current, context.network.current)
 
     def delete_subnet_postcommit(self, context):
         self._ovn_client.delete_subnet(context.current['id'])
@@ -342,10 +377,23 @@ class OVNMechanismDriver(api.MechanismDriver):
         of the current transaction.
         """
         port = context.current
+        if utils.is_lsp_ignored(port):
+            return
         utils.validate_and_get_data_from_binding_profile(port)
         if self._is_port_provisioning_required(port, context.host):
             self._insert_port_provisioning_block(context._plugin_context, port)
 
+        db_rev.create_initial_revision(port['id'],
+                                       ovn_const.TYPE_PORTS,
+                                       context._plugin_context.session)
+
+        # in the case of router ports we also need to
+        # track the creation and update of the LRP OVN objects
+        if utils.is_lsp_router_port(port):
+            db_rev.create_initial_revision(port['id'],
+                                           ovn_const.TYPE_ROUTER_PORTS,
+                                           context._plugin_context.session)
+
     def _is_port_provisioning_required(self, port, host, original_host=None):
         vnic_type = port.get(portbindings.VNIC_TYPE, portbindings.VNIC_NORMAL)
         if vnic_type not in self.supported_vnic_types:
@@ -449,6 +497,14 @@ class OVNMechanismDriver(api.MechanismDriver):
                                                context.original_host):
             self._insert_port_provisioning_block(context._plugin_context, port)
 
+        if utils.is_lsp_router_port(port):
+            # handle the case when an existing port is added to a
+            # logical router so we need to track the creation of the lrp
+            if not utils.is_lsp_router_port(original_port):
+                db_rev.create_initial_revision(port['id'],
+                                               ovn_const.TYPE_ROUTER_PORTS,
+                                               context._plugin_context.session)
+
     def update_port_postcommit(self, context):
         """Update a port.
 
@@ -467,15 +523,9 @@ class OVNMechanismDriver(api.MechanismDriver):
         """
         port = context.current
         original_port = context.original
-        self._ovn_client.update_port(port, original_port)
+        self._ovn_client.update_port(port, port_object=original_port)
         self._notify_dhcp_updated(port['id'])
 
-    # TODO(lucasagomes): We need this because the QOS driver is still
-    # relying on a method called update_port(), delete it after QOS driver
-    # is updated
-    def update_port(self, port, original_port, qos_options=None):
-        self._ovn_client.update_port(port, original_port)
-
     def delete_port_postcommit(self, context):
         """Delete a port.
 
@@ -489,7 +539,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         deleted.
         """
         port = context.current
-        self._ovn_client.delete_port(port)
+        self._ovn_client.delete_port(port['id'], port_object=port)
 
     def bind_port(self, context):
         """Attempt to bind a port.
@@ -540,6 +590,16 @@ class OVNMechanismDriver(api.MechanismDriver):
                       {'port_id': port['id'], 'vnic_type': vnic_type})
             return
 
+        profile = port.get(portbindings.PROFILE)
+        capabilities = []
+        if profile:
+            capabilities = profile.get('capabilities', [])
+        if (vnic_type == portbindings.VNIC_DIRECT and
+           'switchdev' not in capabilities):
+            LOG.debug("Refusing to bind port due to unsupported vnic_type: %s"
+                      "with no switchdev capability", portbindings.VNIC_DIRECT)
+            return
+
         # OVN chassis information is needed to ensure a valid port bind.
         # Collect port binding data and refuse binding if the OVN chassis
         # cannot be found.
@@ -616,7 +676,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         workers, can return a sequence of worker instances.
         """
         # See doc/source/design/ovn_worker.rst for more details.
-        return [ovsdb_monitor.OvnWorker()]
+        return [ovsdb_monitor.OvnWorker(), maintenance.MaintenanceWorker()]
 
     def _update_subport_host_if_needed(self, port_id):
         parent_port = self._ovn_client.get_parent_port(port_id)
@@ -705,8 +765,16 @@ class OVNMechanismDriver(api.MechanismDriver):
         if config.is_ovn_metadata_enabled() and self._sb_ovn:
             # Wait until metadata service has been setup for this port in the
             # chassis it resides.
-            chassis, datapath = (
+            result = (
                 self._sb_ovn.get_logical_port_chassis_and_datapath(port_id))
+            if not result:
+                LOG.warning("Logical port %s doesn't exist in OVN", port_id)
+                return
+            chassis, datapath = result
+            if not chassis:
+                LOG.warning("Logical port %s is not bound to a "
+                            "chassis", port_id)
+                return
             try:
                 n_utils.wait_until_true(
                     lambda: datapath in
