@@ -10,48 +10,68 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_config import cfg
 from oslo_log import log as logging
 
+from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants
+from neutron_lib import context as n_context
+from neutron_lib.db import constants as db_consts
 from neutron_lib.plugins import directory
+from neutron_lib.services.qos import base
+from neutron_lib.services.qos import constants as qos_consts
 
-from neutron import context as n_context
 from neutron.objects.qos import policy as qos_policy
 from neutron.objects.qos import rule as qos_rule
-from neutron.plugins.ml2 import plugin as ml2_plugin
-from neutron.services.qos.notification_drivers import qos_base
-
-from networking_ovn._i18n import _LI
 
+from networking_ovn.common import utils
 
 LOG = logging.getLogger(__name__)
 
+OVN_QOS = 'qos'
+SUPPORTED_RULES = {
+    qos_consts.RULE_TYPE_BANDWIDTH_LIMIT: {
+        qos_consts.MAX_KBPS: {
+            'type:range': [0, db_consts.DB_INTEGER_MAX_VALUE]},
+        qos_consts.MAX_BURST: {
+            'type:range': [0, db_consts.DB_INTEGER_MAX_VALUE]},
+        qos_consts.DIRECTION: {
+            'type:values': [constants.EGRESS_DIRECTION]}
+    },
+}
 
-class OVNQosNotificationDriver(qos_base.QosServiceNotificationDriverBase):
+VIF_TYPES = [portbindings.VIF_TYPE_OVS, portbindings.VIF_TYPE_VHOST_USER]
+VNIC_TYPES = [portbindings.VNIC_NORMAL]
+
+
+class OVNQosNotificationDriver(base.DriverBase):
     """OVN notification driver for QoS."""
 
-    def __init__(self):
-        self._driver_property = None
+    def __init__(self, name='OVNQosDriver',
+                 vif_types=VIF_TYPES,
+                 vnic_types=VNIC_TYPES,
+                 supported_rules=SUPPORTED_RULES,
+                 requires_rpc_notifications=False):
+        super(OVNQosNotificationDriver, self).__init__(
+            name, vif_types, vnic_types, supported_rules,
+            requires_rpc_notifications)
 
-    def get_description(self):
-        return "OVN notification driver"
+    @classmethod
+    def create(cls, plugin_driver):
+        cls._driver = plugin_driver
+        return cls()
 
     @property
-    def _driver(self):
-        if self._driver_property is None:
-            plugin = directory.get_plugin()
-            if isinstance(plugin, ml2_plugin.Ml2Plugin):
-                self._driver_property = \
-                    plugin.mechanism_manager.mech_drivers['ovn'].obj
-        return self._driver_property
+    def is_loaded(self):
+        return OVN_QOS in cfg.CONF.ml2.extension_drivers
 
     def create_policy(self, context, policy):
         # No need to update OVN on create
         pass
 
     def update_policy(self, context, policy):
-        # Call into qos_driver to update the policy
-        self._driver.qos_driver.update_policy(context, policy)
+        # Call into OVN client to update the policy
+        self._driver._ovn_client._qos_driver.update_policy(context, policy)
 
     def delete_policy(self, context, policy):
         # No need to update OVN on delete
@@ -62,7 +82,7 @@ class OVNQosDriver(object):
     """Qos driver for OVN"""
 
     def __init__(self, driver):
-        LOG.info(_LI("Starting OVNQosDriver"))
+        LOG.info("Starting OVNQosDriver")
         super(OVNQosDriver, self).__init__()
         self._driver = driver
         self._plugin_property = None
@@ -73,13 +93,6 @@ class OVNQosDriver(object):
             self._plugin_property = directory.get_plugin()
         return self._plugin_property
 
-    def _is_network_device_port(self, port):
-        device_owner = port.get('device_owner')
-        if (device_owner and
-                device_owner.startswith(constants.DEVICE_OWNER_PREFIXES)):
-            return True
-        return False
-
     def _generate_port_options(self, context, policy_id):
         if policy_id is None:
             return {}
@@ -89,9 +102,9 @@ class OVNQosDriver(object):
         for rule in all_rules:
             if isinstance(rule, qos_rule.QosBandwidthLimitRule):
                 if rule.max_kbps:
-                    options['policing_rate'] = str(rule.max_kbps)
+                    options['qos_max_rate'] = str(rule.max_kbps * 1000)
                 if rule.max_burst_kbps:
-                    options['policing_burst'] = str(rule.max_burst_kbps)
+                    options['qos_burst'] = str(rule.max_burst_kbps * 1000)
         return options
 
     def get_qos_options(self, port):
@@ -99,7 +112,7 @@ class OVNQosDriver(object):
         if 'qos_policy_id' not in port:
             return {}
         # Don't apply qos rules to network devices
-        if self._is_network_device_port(port):
+        if utils.is_network_device_port(port):
             return {}
 
         # Determine if port or network policy should be used
@@ -125,24 +138,20 @@ class OVNQosDriver(object):
             if port_policy_id:
                 continue
             # Don't apply qos rules to network devices
-            if self._is_network_device_port(port):
+            if utils.is_network_device_port(port):
                 continue
-            # Call into mech driver to update port
-            self._driver.update_port(port, port, options)
+            # Call into OVN client to update port
+            self._driver.update_port(port, qos_options=options)
 
-    def update_network(self, network, original_network):
+    def update_network(self, network):
         # Is qos service enabled
         if 'qos_policy_id' not in network:
             return
-        # Was network qos policy changed
-        network_policy_id = network.get('qos_policy_id')
-        old_network_policy_id = original_network.get('qos_policy_id')
-        if network_policy_id == old_network_policy_id:
-            return
 
         # Update the qos options on each network port
         context = n_context.get_admin_context()
-        options = self._generate_port_options(context, network_policy_id)
+        options = self._generate_port_options(
+            context, network['qos_policy_id'])
         self._update_network_ports(context, network.get('id'), options)
 
     def update_policy(self, context, policy):
@@ -157,4 +166,4 @@ class OVNQosDriver(object):
         port_bindings = policy.get_bound_ports()
         for port_id in port_bindings:
             port = self._plugin.get_port(context, port_id)
-            self._driver.update_port(port, port, options)
+            self._driver.update_port(port, qos_options=options)
