@@ -14,8 +14,19 @@ from ovsdbapp.backend.ovs_idl import command
 from ovsdbapp.backend.ovs_idl import idlutils
 
 from networking_ovn._i18n import _
+from networking_ovn.common import constants as ovn_const
+from networking_ovn.common import exceptions as ovn_exc
 from networking_ovn.common import utils
 
+RESOURCE_TYPE_MAP = {
+    ovn_const.TYPE_NETWORKS: 'Logical_Switch',
+    ovn_const.TYPE_PORTS: 'Logical_Switch_Port',
+    ovn_const.TYPE_ROUTERS: 'Logical_Router',
+    ovn_const.TYPE_ROUTER_PORTS: 'Logical_Router_Port',
+    ovn_const.TYPE_FLOATINGIPS: 'NAT',
+    ovn_const.TYPE_SUBNETS: 'DHCP_Options',
+}
+
 
 def _addvalue_to_list(row, column, new_value):
     row.addvalue(column, new_value)
@@ -50,54 +61,35 @@ def get_lsp_dhcp_options_uuids(lsp, lsp_name):
     return uuids
 
 
-class AddLSwitchCommand(command.BaseCommand):
-    def __init__(self, api, name, may_exist, **columns):
-        super(AddLSwitchCommand, self).__init__(api)
+def _add_gateway_chassis(api, txn, lrp_name, val):
+    gateway_chassis = api._tables.get('Gateway_Chassis')
+    if gateway_chassis:
+        prio = len(val)
+        uuid_list = []
+        for chassis in val:
+            gwc_name = '%s_%s' % (lrp_name, chassis)
+            try:
+                gwc = idlutils.row_by_value(api.idl,
+                                            'Gateway_Chassis',
+                                            'name', gwc_name)
+            except idlutils.RowNotFound:
+                gwc = txn.insert(gateway_chassis)
+                gwc.name = gwc_name
+            gwc.chassis_name = chassis
+            gwc.priority = prio
+            prio = prio - 1
+            uuid_list.append(gwc.uuid)
+        return 'gateway_chassis', uuid_list
+    else:
+        chassis = {ovn_const.OVN_GATEWAY_CHASSIS_KEY: val[0]}
+        return 'options', chassis
+
+
+class LSwitchSetExternalIdsCommand(command.BaseCommand):
+    def __init__(self, api, name, ext_ids, if_exists):
+        super(LSwitchSetExternalIdsCommand, self).__init__(api)
         self.name = name
-        self.columns = columns
-        self.may_exist = may_exist
-
-    def run_idl(self, txn):
-        if self.may_exist:
-            lswitch = idlutils.row_by_value(self.api.idl, 'Logical_Switch',
-                                            'name', self.name, None)
-            if lswitch:
-                return
-        row = txn.insert(self.api._tables['Logical_Switch'])
-        row.name = self.name
-        for col, val in self.columns.items():
-            setattr(row, col, val)
-        self.result = row.uuid
-
-    def post_commit(self, txn):
-        self.result = txn.get_insert_uuid(self.result)
-
-
-class DelLSwitchCommand(command.BaseCommand):
-    def __init__(self, api, name, if_exists):
-        super(DelLSwitchCommand, self).__init__(api)
-        self.name = name
-        self.if_exists = if_exists
-
-    def run_idl(self, txn):
-        try:
-            lswitch = idlutils.row_by_value(self.api.idl, 'Logical_Switch',
-                                            'name', self.name)
-        except idlutils.RowNotFound:
-            if self.if_exists:
-                return
-            msg = _("Logical Switch %s does not exist") % self.name
-            raise RuntimeError(msg)
-
-        self.api._tables['Logical_Switch'].rows[lswitch.uuid].delete()
-
-
-class LSwitchSetExternalIdCommand(command.BaseCommand):
-    def __init__(self, api, name, field, value, if_exists):
-        super(LSwitchSetExternalIdCommand, self).__init__(api)
-        self.name = name
-        self.field = field
-        self.value = value
+        self.ext_ids = ext_ids
         self.if_exists = if_exists
 
     def run_idl(self, txn):
@@ -112,9 +104,9 @@ class LSwitchSetExternalIdCommand(command.BaseCommand):
             raise RuntimeError(msg)
 
         lswitch.verify('external_ids')
-
         external_ids = getattr(lswitch, 'external_ids', {})
-        external_ids[self.field] = self.value
+        for key, value in self.ext_ids.items():
+            external_ids[key] = value
         lswitch.external_ids = external_ids
 
 
@@ -306,10 +298,11 @@ class DelLRouterCommand(command.BaseCommand):
 
 
 class AddLRouterPortCommand(command.BaseCommand):
-    def __init__(self, api, name, lrouter, **columns):
+    def __init__(self, api, name, lrouter, may_exist, **columns):
         super(AddLRouterPortCommand, self).__init__(api)
         self.name = name
         self.lrouter = lrouter
+        self.may_exist = may_exist
         self.columns = columns
 
     def run_idl(self, txn):
@@ -323,6 +316,8 @@ class AddLRouterPortCommand(command.BaseCommand):
         try:
             idlutils.row_by_value(self.api.idl, 'Logical_Router_Port',
                                   'name', self.name)
+            if self.may_exist:
+                return
             # The LRP entry with certain name has already exist, raise an
             # exception to notice caller. It's caller's responsibility to
             # call UpdateLRouterPortCommand to get LRP entry processed
@@ -334,6 +329,9 @@ class AddLRouterPortCommand(command.BaseCommand):
             lrouter_port = txn.insert(self.api._tables['Logical_Router_Port'])
             lrouter_port.name = self.name
             for col, val in self.columns.items():
+                if col == 'gateway_chassis':
+                    col, val = _add_gateway_chassis(self.api, txn, self.name,
+                                                    val)
                 setattr(lrouter_port, col, val)
             _addvalue_to_list(lrouter, 'ports', lrouter_port)
 
@@ -358,6 +356,9 @@ class UpdateLRouterPortCommand(command.BaseCommand):
 
         if lrouter_port:
             for col, val in self.columns.items():
+                if col == 'gateway_chassis':
+                    col, val = _add_gateway_chassis(self.api, txn, self.name,
+                                                    val)
                 setattr(lrouter_port, col, val)
             return
 
@@ -390,10 +391,11 @@ class DelLRouterPortCommand(command.BaseCommand):
 
 
 class SetLRouterPortInLSwitchPortCommand(command.BaseCommand):
-    def __init__(self, api, lswitch_port, lrouter_port, if_exists):
+    def __init__(self, api, lswitch_port, lrouter_port, is_gw_port, if_exists):
         super(SetLRouterPortInLSwitchPortCommand, self).__init__(api)
         self.lswitch_port = lswitch_port
         self.lrouter_port = lrouter_port
+        self.is_gw_port = is_gw_port
         self.if_exists = if_exists
 
     def run_idl(self, txn):
@@ -408,6 +410,8 @@ class SetLRouterPortInLSwitchPortCommand(command.BaseCommand):
             raise RuntimeError(msg)
 
         options = {'router-port': self.lrouter_port}
+        if self.is_gw_port:
+            options[ovn_const.OVN_GATEWAY_NAT_ADDRESSES_KEY] = 'router'
         setattr(port, 'options', options)
         setattr(port, 'type', 'router')
         setattr(port, 'addresses', 'router')
@@ -431,7 +435,6 @@ class AddACLCommand(command.BaseCommand):
         row = txn.insert(self.api._tables['ACL'])
         for col, val in self.columns.items():
             setattr(row, col, val)
-        row.external_ids = {'neutron:lport': self.lport}
         _addvalue_to_list(lswitch, 'acls', row.uuid)
 
 
@@ -561,15 +564,18 @@ class UpdateACLsCommand(command.BaseCommand):
         else:
             acl_add_values_dict = {}
             acl_del_objs_dict = {}
-            del_acl_matches = []
+            del_acl_extids = []
             for acl_dict in self.acl_new_values_dict.values():
-                del_acl_matches.append(acl_dict['match'])
+                del_acl_extids.append({acl_dict['match']:
+                                       acl_dict['external_ids']})
             for switch_name, lswitch in lswitch_ovsdb_dict.items():
                 if switch_name not in acl_del_objs_dict:
                     acl_del_objs_dict[switch_name] = []
                 acls = getattr(lswitch, 'acls', [])
                 for acl in acls:
-                    if getattr(acl, 'match') in del_acl_matches:
+                    match = getattr(acl, 'match')
+                    acl_extids = {match: getattr(acl, 'external_ids')}
+                    if acl_extids in del_acl_extids:
                         acl_del_objs_dict[switch_name].append(acl)
         return lswitch_ovsdb_dict, acl_del_objs_dict, acl_add_values_dict
 
@@ -806,11 +812,11 @@ class UpdatePortBindingExtIdsCommand(command.BaseCommand):
 
 
 class AddDHCPOptionsCommand(command.BaseCommand):
-    def __init__(self, api, subnet_id, port_id=None, may_exists=True,
+    def __init__(self, api, subnet_id, port_id=None, may_exist=True,
                  **columns):
         super(AddDHCPOptionsCommand, self).__init__(api)
         self.columns = columns
-        self.may_exists = may_exists
+        self.may_exist = may_exist
         self.subnet_id = subnet_id
         self.port_id = port_id
         self.new_insert = False
@@ -825,7 +831,7 @@ class AddDHCPOptionsCommand(command.BaseCommand):
 
     def run_idl(self, txn):
         row = None
-        if self.may_exists:
+        if self.may_exist:
             row = self._get_dhcp_options_row()
 
         if not row:
@@ -1007,3 +1013,136 @@ class DeleteNatIpFromLRPortPeerOptionsCommand(command.BaseCommand):
         else:
             options['nat-addresses'] = nat_ips
         lport.options = options
+
+
+class CheckRevisionNumberCommand(command.BaseCommand):
+
+    def __init__(self, api, name, resource, resource_type, if_exists):
+        super(CheckRevisionNumberCommand, self).__init__(api)
+        self.name = name
+        self.resource = resource
+        self.resource_type = resource_type
+        self.if_exists = if_exists
+
+    def _get_floatingip(self):
+        # TODO(lucasagomes): We can't use self.api.lookup() because that
+        # method does not introspect map type columns. We could either:
+        # 1. Enhance it to look into maps or, 2. Add a new ``name`` column
+        # to the NAT table so that we can use lookup() just like we do
+        # for other resources
+        for nat in self.api._tables['NAT'].rows.values():
+            if nat.type != 'dnat_and_snat':
+                continue
+            ext_ids = getattr(nat, 'external_ids', {})
+            if ext_ids.get(ovn_const.OVN_FIP_EXT_ID_KEY) == self.name:
+                return nat
+
+        raise idlutils.RowNotFound(
+            table='NAT', col='external_ids', match=self.name)
+
+    def _get_subnet(self):
+        for dhcp in self.api._tables['DHCP_Options'].rows.values():
+            ext_ids = getattr(dhcp, 'external_ids', {})
+            # Ignore ports DHCP Options
+            if ext_ids.get('port_id'):
+                continue
+            if ext_ids.get('subnet_id') == self.name:
+                return dhcp
+
+        raise idlutils.RowNotFound(
+            table='DHCP_Options', col='external_ids', match=self.name)
+
+    def run_idl(self, txn):
+        try:
+            ovn_table = RESOURCE_TYPE_MAP[self.resource_type]
+            # TODO(lucasagomes): After OVS 2.8.2 is released all tables should
+            # have the external_ids column. We can remove this conditional
+            # here by then.
+            if not self.api.is_col_present(ovn_table, 'external_ids'):
+                return
+
+            ovn_resource = None
+            if self.resource_type == ovn_const.TYPE_FLOATINGIPS:
+                ovn_resource = self._get_floatingip()
+            elif self.resource_type == ovn_const.TYPE_SUBNETS:
+                ovn_resource = self._get_subnet()
+            else:
+                ovn_resource = self.api.lookup(ovn_table, self.name)
+        except idlutils.RowNotFound:
+            if self.if_exists:
+                return
+            msg = (_('Failed to check the revision number for %s: Resource '
+                     'does not exist') % self.name)
+            raise RuntimeError(msg)
+
+        external_ids = getattr(ovn_resource, 'external_ids', {})
+        ovn_revision = int(external_ids.get(
+            ovn_const.OVN_REV_NUM_EXT_ID_KEY, -1))
+        neutron_revision = utils.get_revision_number(self.resource,
+                                                     self.resource_type)
+        if ovn_revision > neutron_revision:
+            raise ovn_exc.RevisionConflict(
+                resource_id=self.name, resource_type=self.resource_type)
+
+        ovn_resource.verify('external_ids')
+        ovn_resource.setkey('external_ids', ovn_const.OVN_REV_NUM_EXT_ID_KEY,
+                            str(neutron_revision))
+
+    def post_commit(self, txn):
+        self.result = ovn_const.TXN_COMMITTED
+
+
+class DeleteLRouterExtGwCommand(command.BaseCommand):
+
+    def __init__(self, api, lrouter, if_exists):
+        super(DeleteLRouterExtGwCommand, self).__init__(api)
+        self.lrouter = lrouter
+        self.if_exists = if_exists
+
+    def run_idl(self, txn):
+        # TODO(lucasagomes): Remove this check after OVS 2.8.2 is tagged
+        # (prior to that, the external_ids column didn't exist in this
+        # table).
+        if not self.api.is_col_present('Logical_Router_Static_Route',
+                                       'external_ids'):
+            return
+
+        try:
+            lrouter = idlutils.row_by_value(self.api.idl, 'Logical_Router',
+                                            'name', self.lrouter)
+        except idlutils.RowNotFound:
+            if self.if_exists:
+                return
+            msg = _("Logical Router %s does not exist") % self.lrouter
+            raise RuntimeError(msg)
+
+        lrouter.verify('static_routes')
+        static_routes = getattr(lrouter, 'static_routes', [])
+        for route in static_routes:
+            external_ids = getattr(route, 'external_ids', {})
+            if ovn_const.OVN_ROUTER_IS_EXT_GW in external_ids:
+                _delvalue_from_list(lrouter, 'static_routes', route)
+                route.delete()
+                break
+
+        lrouter.verify('nat')
+        nats = getattr(lrouter, 'nat', [])
+        for nat in nats:
+            if nat.type != 'snat':
+                continue
+            _delvalue_from_list(lrouter, 'nat', nat)
+            nat.delete()
+
+        lrouter_ext_ids = getattr(lrouter, 'external_ids', {})
+        gw_port_id = lrouter_ext_ids.get(ovn_const.OVN_GW_PORT_EXT_ID_KEY)
+        if not gw_port_id:
+            return
+
+        try:
+            lrouter_port = idlutils.row_by_value(
+                self.api.idl, 'Logical_Router_Port', 'name',
+                utils.ovn_lrouter_port_name(gw_port_id))
+        except idlutils.RowNotFound:
+            return
+
+        _delvalue_from_list(lrouter, 'ports', lrouter_port)
