@@ -12,10 +12,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import atexit
-import threading
-
-from eventlet import greenthread
 from neutron.common import config
 from neutron_lib.plugins import constants
 from neutron_lib.plugins import directory
@@ -24,11 +20,12 @@ from neutron_lib import worker
 from oslo_log import log
 from ovs.stream import Stream
 from ovsdbapp.backend.ovs_idl import connection
+from ovsdbapp.backend.ovs_idl import event as row_event
 from ovsdbapp.backend.ovs_idl import idlutils
-from six.moves import queue
+from ovsdbapp import event
 
 from networking_ovn.common import config as ovn_config
-from networking_ovn.ovsdb import row_event
+from networking_ovn.common import utils
 
 LOG = log.getLogger(__name__)
 
@@ -54,7 +51,7 @@ class ChassisEvent(row_event.RowEvent):
             phy_nets = list(mapping_dict)
 
         self.driver.update_segment_host_mapping(host, phy_nets)
-        if ovn_config.is_ovn_l3():
+        if utils.is_ovn_l3(self.l3_plugin):
             self.l3_plugin.schedule_unhosted_gateways()
 
 
@@ -137,76 +134,10 @@ class LogicalSwitchPortUpdateDownEvent(row_event.RowEvent):
         self.driver.set_port_status_down(row.name)
 
 
-class OvnDbNotifyHandler(object):
-
-    STOP_EVENT = ("STOP", None, None, None)
-
+class OvnDbNotifyHandler(event.RowEventHandler):
     def __init__(self, driver):
+        super(OvnDbNotifyHandler, self).__init__()
         self.driver = driver
-        self.__watched_events = set()
-        self.__lock = threading.Lock()
-        self.notifications = queue.Queue()
-        self.notify_thread = greenthread.spawn_n(self.notify_loop)
-        atexit.register(self.shutdown)
-
-    def matching_events(self, event, row, updates):
-        with self.__lock:
-            return tuple(t for t in self.__watched_events
-                         if t.matches(event, row, updates))
-
-    def watch_event(self, event):
-        with self.__lock:
-            self.__watched_events.add(event)
-
-    def watch_events(self, events):
-        with self.__lock:
-            for event in events:
-                self.__watched_events.add(event)
-
-    def unwatch_event(self, event):
-        with self.__lock:
-            try:
-                self.__watched_events.remove(event)
-            except KeyError:
-                # For ONETIME events, they should normally clear on their own
-                pass
-
-    def unwatch_events(self, events):
-        with self.__lock:
-            for event in events:
-                try:
-                    self.__watched_events.remove(event)
-                except KeyError:
-                    # For ONETIME events, they should normally clear on
-                    # their own
-                    pass
-
-    def shutdown(self):
-        self.notifications.put(OvnDbNotifyHandler.STOP_EVENT)
-
-    def notify_loop(self):
-        while True:
-            try:
-                match, event, row, updates = self.notifications.get()
-                if (not isinstance(match, row_event.RowEvent) and
-                        (match, event, row, updates) == (
-                            OvnDbNotifyHandler.STOP_EVENT)):
-                    self.notifications.task_done()
-                    break
-                match.run(event, row, updates)
-                if match.ONETIME:
-                    self.unwatch_event(match)
-                self.notifications.task_done()
-            except Exception:
-                # If any unexpected exception happens we don't want the
-                # notify_loop to exit.
-                LOG.exception('Unexpected exception in notify_loop')
-
-    def notify(self, event, row, updates=None):
-        matching = self.matching_events(
-            event, row, updates)
-        for match in matching:
-            self.notifications.put((match, event, row, updates))
 
 
 class BaseOvnIdl(connection.OvsdbIdl):
@@ -224,6 +155,7 @@ class BaseOvnSbIdl(connection.OvsdbIdl):
         _check_and_set_ssl_files(schema_name)
         helper = idlutils.get_schema_helper(connection_string, schema_name)
         helper.register_table('Chassis')
+        helper.register_table('Encap')
         if ovn_config.is_ovn_metadata_enabled():
             helper.register_table('Port_Binding')
             helper.register_table('Datapath_Binding')
@@ -258,7 +190,7 @@ class OvnIdl(BaseOvnIdl):
     def notify(self, event, row, updates=None):
         # Do not handle the notification if the event lock is requested,
         # but not granted by the ovsdb-server.
-        if (self.is_lock_contended and not self.has_lock):
+        if self.is_lock_contended:
             return
         self.notify_handler.notify(event, row, updates)
 
@@ -316,6 +248,7 @@ class OvnSbIdl(OvnIdl):
         _check_and_set_ssl_files(schema_name)
         helper = idlutils.get_schema_helper(connection_string, schema_name)
         helper.register_table('Chassis')
+        helper.register_table('Encap')
         if ovn_config.is_ovn_metadata_enabled():
             helper.register_table('Port_Binding')
             helper.register_table('Datapath_Binding')
