@@ -12,11 +12,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import os
 import time
 
 import fixtures
 import mock
-from neutron.plugins.ml2 import config
+from neutron.conf.plugins.ml2 import config
 from neutron.plugins.ml2.drivers import type_geneve  # noqa
 from neutron.tests.unit.plugins.ml2 import test_plugin
 from neutron_lib.plugins import constants
@@ -26,14 +27,21 @@ from oslo_log import log
 from oslo_utils import uuidutils
 from ovsdbapp.backend.ovs_idl import command
 from ovsdbapp.backend.ovs_idl import connection
-from ovsdbapp.backend.ovs_idl import transaction
 
+# Load all the models to register them into SQLAlchemy metadata before using
+# the SqlFixture
+from networking_ovn.db import models  # noqa
 from networking_ovn.ovsdb import impl_idl_ovn
 from networking_ovn.ovsdb import ovsdb_monitor
+from networking_ovn.tests import base
 from networking_ovn.tests.functional.resources import process
 
 LOG = log.getLogger(__name__)
 
+# This is the directory from which infra fetches log files for functional tests
+DEFAULT_LOG_DIR = os.path.join(os.environ.get('OS_LOG_PATH', '/tmp'),
+                               'dsvm-functional-logs')
+
 
 class AddFakeChassisCommand(command.BaseCommand):
     """Add a fake chassis in OVN SB DB for functional test."""
@@ -56,6 +64,21 @@ class AddFakeChassisCommand(command.BaseCommand):
             setattr(row, col, val)
 
 
+class ConnectionFixture(fixtures.Fixture):
+    def __init__(self, idl=None, constr=None, schema=None, timeout=60):
+        self.idl = idl or ovsdb_monitor.BaseOvnIdl.from_server(
+            constr, schema)
+        self.connection = connection.Connection(
+            idl=self.idl, timeout=timeout)
+
+    def _setUp(self):
+        self.addCleanup(self.stop)
+        self.connection.start()
+
+    def stop(self):
+        self.connection.stop()
+
+
 class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
 
     # Please see networking_ovn/tests/contrib/gate_hook.sh.
@@ -78,6 +101,9 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
                                      group='ml2_type_geneve')
 
         super(TestOVNFunctionalBase, self).setUp()
+        base.setup_test_logging(
+            cfg.CONF, DEFAULT_LOG_DIR, "%s.txt" % self.id())
+
         mm = directory.get_plugin().mechanism_manager
         self.mech_driver = mm.mech_drivers['ovn'].obj
         self.l3_plugin = directory.get_plugin(constants.L3)
@@ -85,26 +111,10 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
         self.ovn_worker = ovn_worker
         self._start_ovsdb_server_and_idls()
 
-    def tearDown(self):
-        # Set Mock() to idl to avoid SSL file access errors.
-        # This is because, destroying temporary directory containing SSL files
-        # is earlier than terminating thread in Connection() object that
-        # the errors are likely to occur in the short period of time.
-        # However, Connection() does not provider a stop method for run(), we
-        # replace idl with Mock() to avoid accessing.
-        if self._ovsdb_protocol == 'ssl':
-            impl_idl_ovn.OvsdbNbOvnIdl.ovsdb_connection.idl = mock.Mock()
-            impl_idl_ovn.OvsdbSbOvnIdl.ovsdb_connection.idl = mock.Mock()
-            self.monitor_nb_idl_con.idl = mock.Mock()
-            self.monitor_sb_idl_con.idl = mock.Mock()
-        # Need to set OvsdbNbOvnIdl.ovsdb_connection and
-        # OvsdbSbOvnIdl.ovsdb_connection to None.
-        # This is because, when the test worker runs the next functional test
-        # case, the plugin will try to use the ovsdb_connection from the
-        # previous test case and will cause the test case to fail.
-        impl_idl_ovn.OvsdbNbOvnIdl.ovsdb_connection = None
-        impl_idl_ovn.OvsdbSbOvnIdl.ovsdb_connection = None
-        super(TestOVNFunctionalBase, self).tearDown()
+    def get_additional_service_plugins(self):
+        p = super(TestOVNFunctionalBase, self).get_additional_service_plugins()
+        p.update({'revision_plugin_name': 'revisions'})
+        return p
 
     @property
     def _ovsdb_protocol(self):
@@ -118,7 +128,7 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
         # Start 2 ovsdb-servers one each for OVN NB DB and OVN SB DB
         # ovsdb-server with OVN SB DB can be used to test the chassis up/down
         # events.
-        self.ovsdb_server_mgr = self.useFixture(
+        mgr = self.ovsdb_server_mgr = self.useFixture(
             process.OvsdbServer(self.temp_dir, self.OVS_INSTALL_SHARE_PATH,
                                 ovn_nb_db=True, ovn_sb_db=True,
                                 protocol=self._ovsdb_protocol))
@@ -151,13 +161,10 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
         #     ML2 OVN driver scope to test scenarios like ovn_nb_sync.
         while num_attempts < 3:
             try:
-                _idlnb = ovsdb_monitor.BaseOvnIdl.from_server(
-                    self.ovsdb_server_mgr.get_ovsdb_connection_path(),
-                    'OVN_Northbound')
-                self.monitor_nb_idl_con = connection.Connection(
-                    idl=_idlnb, timeout=60)
-                self.monitor_nb_idl_con.start()
-                self.monitor_nb_db_idl = self.monitor_nb_idl_con.idl
+                con = self.useFixture(ConnectionFixture(
+                    constr=mgr.get_ovsdb_connection_path(),
+                    schema='OVN_Northbound')).connection
+                self.nb_api = impl_idl_ovn.OvsdbNbOvnIdl(con)
                 break
             except Exception:
                 LOG.exception("Error connecting to the OVN_Northbound DB")
@@ -172,13 +179,10 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
         #  - Update chassis columns etc.
         while num_attempts < 3:
             try:
-                _idlsb = ovsdb_monitor.BaseOvnIdl.from_server(
-                    self.ovsdb_server_mgr.get_ovsdb_connection_path('sb'),
-                    'OVN_Southbound')
-                self.monitor_sb_idl_con = connection.Connection(
-                    idl=_idlsb, timeout=60)
-                self.monitor_sb_idl_con.start()
-                self.monitor_sb_db_idl = self.monitor_sb_idl_con.idl
+                con = self.useFixture(ConnectionFixture(
+                    constr=mgr.get_ovsdb_connection_path('sb'),
+                    schema='OVN_Southbound')).connection
+                self.sb_api = impl_idl_ovn.OvsdbSbOvnIdl(con)
                 break
             except Exception:
                 LOG.exception("Error connecting to the OVN_Southbound DB")
@@ -191,52 +195,48 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
             cfg.CONF.set_override('neutron_sync_mode', 'off', 'ovn')
         trigger.im_class.__name__ = 'trigger'
 
+        self.addCleanup(self.stop)
+
         # mech_driver.post_fork_initialize creates the IDL connections
         self.mech_driver.post_fork_initialize(mock.ANY, mock.ANY, trigger)
 
-    def nb_idl_transaction(self, fake_api, check_error=False, log_errors=True,
-                           **kwargs):
-        return transaction.Transaction(fake_api, self.monitor_nb_idl_con, 60,
-                                       check_error, log_errors)
-
-    def sb_idl_transaction(self, fake_api, check_error=False, log_errors=True,
-                           **kwargs):
-        return transaction.Transaction(fake_api, self.monitor_sb_idl_con, 60,
-                                       check_error, log_errors)
+    def stop(self):
+        if self.ovn_worker:
+            self.mech_driver.nb_synchronizer.stop()
+            self.mech_driver.sb_synchronizer.stop()
+        self.mech_driver._nb_ovn.ovsdb_connection.stop()
+        self.mech_driver._sb_ovn.ovsdb_connection.stop()
 
     def restart(self):
+        self.stop()
+        # The OVN sync test starts its own synchronizers...
+        self.l3_plugin._nb_ovn_idl.ovsdb_connection.stop()
+        self.l3_plugin._sb_ovn_idl.ovsdb_connection.stop()
+        # Stop our monitor connections
+        self.nb_api.ovsdb_connection.stop()
+        self.sb_api.ovsdb_connection.stop()
+
         if self.ovsdb_server_mgr:
             self.ovsdb_server_mgr.stop()
 
-        if self._ovsdb_protocol == 'ssl':
-            impl_idl_ovn.OvsdbNbOvnIdl.ovsdb_connection.idl = mock.Mock()
-            impl_idl_ovn.OvsdbSbOvnIdl.ovsdb_connection.idl = mock.Mock()
-            self.monitor_nb_idl_con.idl = mock.Mock()
-            self.monitor_sb_idl_con.idl = mock.Mock()
-        impl_idl_ovn.OvsdbNbOvnIdl.ovsdb_connection = None
-        impl_idl_ovn.OvsdbSbOvnIdl.ovsdb_connection = None
         self.mech_driver._nb_ovn = None
         self.mech_driver._sb_ovn = None
         self.l3_plugin._nb_ovn_idl = None
         self.l3_plugin._sb_ovn_idl = None
-        self.monitor_nb_idl_con = None
-        self.monitor_sb_idl_con = None
+        self.nb_api.ovsdb_connection = None
+        self.sb_api.ovsdb_connection = None
 
         self._start_ovsdb_server_and_idls()
 
     def add_fake_chassis(self, host, physical_nets=None, external_ids=None):
         physical_nets = physical_nets or []
         external_ids = external_ids or {}
-        fake_api = mock.MagicMock()
-        fake_api.idl = self.monitor_sb_db_idl
-        fake_api._tables = self.monitor_sb_db_idl.tables
 
         bridge_mapping = ",".join(["%s:br-provider%s" % (phys_net, i)
                                   for i, phys_net in enumerate(physical_nets)])
         name = uuidutils.generate_uuid()
-        with self.sb_idl_transaction(fake_api, check_error=True) as txn:
-            external_ids['ovn-bridge-mappings'] = bridge_mapping
-            txn.add(AddFakeChassisCommand(fake_api, name, "172.24.4.10",
-                                          external_ids=external_ids,
-                                          hostname=host))
+        external_ids['ovn-bridge-mappings'] = bridge_mapping
+        self.sb_api.chassis_add(
+            name, ['geneve'], '172.24.4.10', external_ids=external_ids,
+            hostname=host).execute(check_error=True)
         return name
