@@ -48,20 +48,26 @@ class TestACLs(base.TestCase):
             'ovsdbapp.backend.ovs_idl.idlutils.row_by_value',
             lambda *args, **kwargs: mock.MagicMock())
         patcher.start()
+        mock.patch(
+            "networking_ovn.common.acl._acl_columns_name_severity_supported",
+            return_value=True
+        ).start()
 
     def test_drop_all_ip_traffic_for_port(self):
         acls = ovn_acl.drop_all_ip_traffic_for_port(self.fake_port)
         acl_to_lport = {'action': 'drop', 'direction': 'to-lport',
                         'external_ids': {'neutron:lport':
                                          self.fake_port['id']},
-                        'log': False, 'lport': self.fake_port['id'],
+                        'log': False, 'name': [], 'severity': [],
+                        'lport': self.fake_port['id'],
                         'lswitch': 'neutron-network_id1',
                         'match': 'outport == "fake_port_id1" && ip',
                         'priority': 1001}
         acl_from_lport = {'action': 'drop', 'direction': 'from-lport',
                           'external_ids': {'neutron:lport':
                                            self.fake_port['id']},
-                          'log': False, 'lport': self.fake_port['id'],
+                          'log': False, 'name': [], 'severity': [],
+                          'lport': self.fake_port['id'],
                           'lswitch': 'neutron-network_id1',
                           'match': 'inport == "fake_port_id1" && ip',
                           'priority': 1001}
@@ -83,7 +89,8 @@ class TestACLs(base.TestCase):
                                    self.fake_subnet['cidr'])
         acl_to_lport = {'action': 'allow', 'direction': 'to-lport',
                         'external_ids': {'neutron:lport': 'fake_port_id1'},
-                        'log': False, 'lport': 'fake_port_id1',
+                        'log': False, 'name': [], 'severity': [],
+                        'lport': 'fake_port_id1',
                         'lswitch': 'neutron-network_id1',
                         'match': expected_match_to_lport, 'priority': 1002}
         expected_match_from_lport = (
@@ -93,7 +100,8 @@ class TestACLs(base.TestCase):
         ) % (self.fake_port['id'], self.fake_subnet['cidr'])
         acl_from_lport = {'action': 'allow', 'direction': 'from-lport',
                           'external_ids': {'neutron:lport': 'fake_port_id1'},
-                          'log': False, 'lport': 'fake_port_id1',
+                          'log': False, 'name': [], 'severity': [],
+                          'lport': 'fake_port_id1',
                           'lswitch': 'neutron-network_id1',
                           'match': expected_match_from_lport, 'priority': 1002}
         self.assertEqual(1, len(ovn_dhcp_acls))
@@ -113,14 +121,17 @@ class TestACLs(base.TestCase):
                           'lport': 'port-id',
                           'priority': ovn_const.ACL_PRIORITY_ALLOW,
                           'action': ovn_const.ACL_ACTION_ALLOW_RELATED,
-                          'log': False,
+                          'log': False, 'name': [], 'severity': [],
                           'direction': direction,
                           'match': match,
-                          'external_ids': {'neutron:lport': 'port-id'}},
+                          'external_ids': {
+                              'neutron:lport': 'port-id',
+                              'neutron:security_group_rule_id': 'sgr_id'}},
                          acl)
 
     def test_add_sg_rule_acl_for_port_remote_ip_prefix(self):
-        sg_rule = {'direction': 'ingress',
+        sg_rule = {'id': 'sgr_id',
+                   'direction': 'ingress',
                    'ethertype': 'IPv4',
                    'remote_group_id': None,
                    'remote_ip_prefix': '1.1.1.0/24',
@@ -136,7 +147,8 @@ class TestACLs(base.TestCase):
                                             match)
 
     def test_add_sg_rule_acl_for_port_remote_group(self):
-        sg_rule = {'direction': 'ingress',
+        sg_rule = {'id': 'sgr_id',
+                   'direction': 'ingress',
                    'ethertype': 'IPv4',
                    'remote_group_id': 'sg1',
                    'remote_ip_prefix': None,
@@ -288,10 +300,10 @@ class TestACLs(base.TestCase):
         ports = [port1, port2]
         aclport1_new = {'priority': 1002, 'direction': 'to-lport',
                         'match': 'outport == %s && ip4 && icmp4' %
-                        (port1['id'])}
+                        (port1['id']), 'external_ids': {}}
         aclport2_new = {'priority': 1002, 'direction': 'to-lport',
                         'match': 'outport == %s && ip4 && icmp4' %
-                        (port2['id'])}
+                        (port2['id']), 'external_ids': {}}
         acls_new_dict = {'%s' % (port1['id']): aclport1_new,
                          '%s' % (port2['id']): aclport2_new}
 
@@ -311,11 +323,12 @@ class TestACLs(base.TestCase):
 
         # test for deleting existing acls
         acl1 = mock.Mock(
-            match='outport == port-id1 && ip4 && icmp4')
+            match='outport == port-id1 && ip4 && icmp4', external_ids={})
         acl2 = mock.Mock(
-            match='outport == port-id2 && ip4 && icmp4')
+            match='outport == port-id2 && ip4 && icmp4', external_ids={})
         acl3 = mock.Mock(
-            match='outport == port-id1 && ip4 && (ip4.src == fake_ip)')
+            match='outport == port-id1 && ip4 && (ip4.src == fake_ip)',
+            external_ids={})
         lswitch_obj = mock.Mock(
             name='neutron-lswitch-1', acls=[acl1, acl2, acl3])
         with mock.patch('ovsdbapp.backend.ovs_idl.idlutils.row_by_value',
@@ -632,7 +645,7 @@ class TestACLs(base.TestCase):
                         return_value=False):
             acl_list = ovn_acl.add_acls(self.plugin,
                                         self.admin_context,
-                                        port, {}, {})
+                                        port, {}, {}, self.driver._ovn)
             self.assertEqual([], acl_list)
 
             ovn_acl.update_acls_for_security_group(self.plugin,
